#! /usr/bin/env python2.5

from __future__ import with_statement

from boushi import Boushi
from bzv.infra import Infra
from bzv.storagecomponent import BZVStorageComponent as StorageComponent
from os import makedirs
from os.path import isdir, join
from re import compile
from sys import exit
from time import ctime, time
from urllib import urlopen

global infra

HEX_TO_CHAR = lambda x:chr(int(x.group(1),16))
REVERSE_BAD_CHARS = compile('%([0-9A-F]{2})')
def unescapeName(name):
    return REVERSE_BAD_CHARS.sub(HEX_TO_CHAR, name)

def writeBatch(sessionDirectory, number, batch):
    if not isdir(sessionDirectory):
        makedirs(sessionDirectory)

    with open(join(sessionDirectory, 'batch_%0.15d' % number), 'w') as fd:
         fd.write('\n'.join(batch))
    return True

def createBatches(storage, repository, sessionDirectory, batchSize, reindexPartName):
    print "createBatches"
    currentBatch = 0
    batch = []
    identifiersFound = False
    #FIXME implementeren met storagecomponent?
    #db = StorageComponent(storage)
    #for recordIdentifier in db.listIdentifiers(partName=reindexPartName, identifierPrefix=repository):
    #    print recordIdentifier
    #    batch.append(identifier)
    ## is korter, bovendien geen check op deleted nodig want dat doet listIdentifiers inmiddels ook al
    db = Boushi(storage)
    docs = db.ls(repository)
    for doc in docs:
        files = db.find(doc)
        deletedPath = doc + "/deleted"
        if deletedPath in files:
            print "deleted"
            continue
        repository, identifier = doc.split("/")
        recordIdentifier = repository + ':' + unescapeName(identifier)
        batch.append(recordIdentifier)
        if len(batch) == batchSize:
            identifiersFound = writeBatch(sessionDirectory, currentBatch, batch)
            currentBatch += 1
            batch = []
    additionalBatch = 0
    if batch != []:
        identifiersFound = writeBatch(sessionDirectory, currentBatch, batch)
        additionalBatch = 1
    if not identifiersFound:
        return 0
    return currentBatch + additionalBatch

def main(bzvConfigFile, session='', batchSize="200", repository=''):
    global infra
    infra = Infra('Starting Reindexer',bzvConfigFile=bzvConfigFile)
    configRoot = infra.configRoot
    config = configRoot.getSectionDict('system')
    storage = join(config['databasepath'], "storage")
    sruUpdatePortNumber = config['sruupdateportnumber']
    reindexIdentifierBatchPath = config['reindexidentifierbatchpath']
    reindexPartName = config['reindexpartname']

    log = open(join(reindexIdentifierBatchPath, "log") , 'a')
    log.write(ctime() + " start of " + session + '\n')
    sessionDirectory = join(reindexIdentifierBatchPath, session)
    numberOfBatches = 0
    if repository:
        numberOfBatches = createBatches(storage, repository, sessionDirectory, int(batchSize), reindexPartName)
        if numberOfBatches == 0: 
            log.write(ctime() + " nothing to do\n")
            log.write(ctime() + " end of " + session + '\n')
            return(0)

    url = "http://localhost:" + sruUpdatePortNumber + "/reindex?session=" + session + "&batchsize=" + batchSize
    if numberOfBatches == 0:
        response = urlopen(url)
        for line in response.readlines():
            if line.startswith("=batches"): 
                print line
                line.rstrip('\n')
                numberOfBatches = int(line.split(": ")[1])
                print "numberOfBatches=" + str(numberOfBatches)
    if numberOfBatches > 0:
        t0 = time()
        total = 0
        loop = 0
        while loop < numberOfBatches:
            response = urlopen(url)
            for line in response.readlines():
                print line,
                if line.startswith("+"): 
                    total += 1
                    print total
                    continue
                if line.startswith("!"):
                    log.write(ctime() + line)
                    print line,
                    continue
                if line.startswith == "=batches left: 0":
                     break
            loop += 1
        print total
        t1 = time()
        delta = t1-t0
        print delta
        avg = total/delta
        print avg
        log.write(ctime() + " time in seconds " + str(delta) + '\n')
        log.write(ctime() + " " + str(total) + " records indexed\n")
        log.write(ctime() + " # records per sec " + str(avg) + '\n')
    else: log.write(ctime() + " nothing to do\n")
    log.write(ctime() + " end of " + session + '\n')
    log.close()
    exit(0)

if __name__ == '__main__':
    from sys import argv
    usage = "Usage: reindexer.py /etc/bzv/bzv.cfg session=session [batchsize=batchsize] [repository=repository]"
    length = len(argv)
    if length < 3:
        print usage
        exit(1)

    bzvConfigFile = argv[1]
    args = argv[2:]
    batchSize = "200"
    repository = ''
    for arg in args:
        key, value = arg.split('=')
        if key == "session": session = value
        elif key == "batchsize": batchSize = str(value)
        elif key == "repository": repository = value 
    if not session:
        print usage
        exit(1)
    main(bzvConfigFile, session=session, batchSize=batchSize, repository=repository)
